/**
 * 
 */
package stream.dashboard;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JPanel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import stream.data.Statistics;

/**
 * @author chris
 * 
 */
public class MatrixChart extends JPanel {

	static Logger log = LoggerFactory.getLogger(MatrixChart.class);

	/** The unique class ID */
	private static final long serialVersionUID = -5309535228880758642L;

	final static Color LINE_COLOR = new Color(0xe0, 0xe0, 0xe0);
	Set<String> columns = new TreeSet<String>();
	Set<String> rows = new TreeSet<String>();
	Statistics switches = new Statistics();
	int border = 25;
	int borderx = border;
	int bordery = border;
	double total = 1.0d;
	List<String> history = new ArrayList<String>();

	public MatrixChart() {
		setBackground(Color.WHITE);
		setBorder(null);
	}

	public void reset() {
		total = 1.0;
		switches.clear();
		rows.clear();
		columns.clear();
	}

	public void paint(Graphics g) {
		super.paint(g);

		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		log.debug("Switch matrix has dimension {} x {}", rows.size(),
				columns.size());

		// the size of each (square) tile
		Double baseSize = 28.0;

		String mrk = null;
		for (String k : rows) {
			if (mrk == null || mrk.length() < k.length()) {
				mrk = k;
			}
		}
		log.debug("Longest row-key is: {}", mrk);
		if (mrk == null)
			mrk = "XXXXXXXX";
		Rectangle2D r2 = g.getFontMetrics().getStringBounds(mrk, g);
		borderx = (int) (r2.getWidth() + 5);

		String mck = null;
		for (String k : rows) {
			if (mck == null || mck.length() < k.length()) {
				mck = k;
			}
		}

		if (mck == null)
			mck = "         ";

		if (g.getFontMetrics() == null)
			return;

		r2 = g.getFontMetrics().getStringBounds(mck, g);
		bordery = (int) (r2.getWidth() * 0.4 + 5);
		log.debug("Longest col-key is: {}", mck);
		log.debug("borderx: {}, bordery: {}", borderx, bordery);

		Font f = g.getFont().deriveFont(9.0f);
		g.setFont(f);

		int row = 0;
		int col = 0;
		for (String rowKey : rows) {
			for (String colKey : columns) {
				Color orig = g.getColor();
				g.setColor(Color.LIGHT_GRAY);
				Double cx = borderx + baseSize * col + (0.5 * baseSize);
				Double cy = bordery + baseSize * row + (0.5 * baseSize);

				if (col == 0) {
					g2.setColor(LINE_COLOR);
					g2.drawLine(borderx, cy.intValue(),
							borderx + baseSize.intValue() * columns.size(),
							cy.intValue());
					g.setColor(Color.DARK_GRAY);
					g.drawString(rowKey, 5, cy.intValue() + 4);
				}

				if (row == 0) {
					g.setColor(LINE_COLOR);
					g2.drawLine(cx.intValue(), bordery, cx.intValue(), bordery
							+ baseSize.intValue() * rows.size());
					AffineTransform origt = g2.getTransform();
					g.setColor(Color.DARK_GRAY);
					g2.translate(cx.intValue(), bordery - 5);
					g2.rotate(-Math.PI / 8);
					g2.drawString(colKey, 0, 0);
					g2.setTransform(origt);
				}
				g.setColor(orig);

				String key = rowKey + ":" + colKey;
				Double value = switches.get(key);
				if (value != null) {
					Double size = getSize(baseSize, key, value);
					g.setColor(Color.GRAY);
					// g.drawRect(bordery + baseSize.intValue() * row, borderx
					// + baseSize.intValue() * col, baseSize.intValue(),
					// baseSize.intValue());

					int halfx = (size.intValue() / 2);
					int halfy = (size.intValue() / 2);

					g.setColor(Color.pink);

					if (ageInSeconds(key) < 10) {
						g.setColor(Color.RED);
					}

					g.fillOval(cx.intValue() - halfx, cy.intValue() - halfy,
							size.intValue(), size.intValue());
				}
				g.setColor(Color.black);
				col++;
			}
			row++;
			col = 0;
		}
	}

	protected Double ageInSeconds(String key) {
		Double ageInSeconds = (System.currentTimeMillis() - switches.get(key)) / 1000.0d;
		return ageInSeconds;
	}

	protected Double getSize(Double baseSize, String key, Double value) {
		Double ageInSeconds = ageInSeconds(key);
		if (ageInSeconds <= 0.0)
			ageInSeconds = 10.0;
		log.info("Key '{}' is {} seconds old...", key, ageInSeconds);

		value = 10.0 - ageInSeconds;

		return Math.max(3.0d, 0.1 * value * baseSize);
	}

	public void add(String from, String to) {
		Long time = System.currentTimeMillis();
		rows.add(from);
		columns.add(to);
		String key = from + ":" + to;
		switches.add(key, time.doubleValue());
		while (history.size() > 10)
			history.remove(0);
		history.add(key);
		total += 0.1;
		repaint();
	}

	public void tick() {
		total += 1.0d;
		repaint();
	}
}