/**
 * 
 */
package stream.dashboard;

import java.awt.BorderLayout;
import java.awt.Color;
import java.io.Serializable;
import java.util.Set;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PiePlot;

import stream.Data;
import stream.ProcessContext;
import stream.data.Statistics;
import stream.util.KeyFilter;

/**
 * @author chris
 * 
 */
public class Pie extends Widget {

	/** The unique class ID */
	private static final long serialVersionUID = -3094354310354207972L;
	String[] keys;
	PieChart chart;
	boolean aggregate = true;

	public Pie() {
		chart = new PieChart();
	}

	/**
	 * @see stream.dashboard.Widget#init(stream.ProcessContext)
	 */
	@Override
	public void init(ProcessContext ctx) throws Exception {
		super.init(ctx);
		PiePlot plot = chart.getPlot();
		plot.setOutlineVisible(false);
		JFreeChart jfc = new JFreeChart(plot);
		jfc.setBorderVisible(false);
		jfc.setBackgroundPaint(Color.WHITE);
		jfc.removeLegend();
		ChartPanel cp = new ChartPanel(jfc);
		cp.setBorder(null);
		cp.setBackground(Color.WHITE);
		this.content.add(cp, BorderLayout.CENTER);
	}

	/**
	 * @see stream.dashboard.Widget#process(stream.Data)
	 */
	@Override
	public Data process(Data input) {

		if (input == null || keys == null || keys.length == 0)
			return input;

		Set<String> selected = KeyFilter.select(input, keys);
		Statistics st = new Statistics();
		for (String key : selected) {
			Double value = 0.0d;
			Serializable val = input.get(key);
			if (val != null && val instanceof Number) {
				value = ((Number) val).doubleValue();
			}
			st.put(key, value);
		}

		if (!st.isEmpty()) {
			if (aggregate)
				chart.add(st);
			else
				chart.set(st);
		}

		return input;
	}

	/**
	 * @return the keys
	 */
	public String[] getKeys() {
		return keys;
	}

	/**
	 * @param keys
	 *            the keys to set
	 */
	public void setKeys(String[] keys) {
		this.keys = keys;
	}

	/**
	 * @return the aggregate
	 */
	public boolean isAggregate() {
		return aggregate;
	}

	/**
	 * @param aggregate
	 *            the aggregate to set
	 */
	public void setAggregate(boolean aggregate) {
		this.aggregate = aggregate;
	}

	/**
	 * @return the chart
	 */
	public PieChart getChart() {
		return chart;
	}
}