package stream.dashboard;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.jfree.data.DomainOrder;
import org.jfree.data.general.DatasetChangeEvent;
import org.jfree.data.general.DatasetChangeListener;
import org.jfree.data.general.DatasetGroup;
import org.jfree.data.xy.TableXYDataset;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import stream.data.Statistics;
import stream.statistics.StatisticsHistory;

public class TimelineDatasetAdapter implements TableXYDataset {

	static Logger log = LoggerFactory.getLogger(TimelineDatasetAdapter.class);
	final List<DatasetChangeListener> listener = new LinkedList<DatasetChangeListener>();
	DatasetGroup group;
	StatisticsHistory timeline;

	LinkedList<String> seriesKeys = new LinkedList<String>();

	public TimelineDatasetAdapter(StatisticsHistory timeline) {
		this.timeline = timeline;
		setSeriesFilter(".*");
	}

	public Date getStart() {
		return new Date(timeline.mapTimestamp(System.currentTimeMillis()
				- timeline.getStepSize()));
	}

	public Date getEnd() {
		return new Date(timeline.mapTimestamp(System.currentTimeMillis()));
	}

	public void setSeriesFilter(String filter) {

		seriesKeys.clear();
		//
		// for (String key : timeline.getSeriesKeys()) {
		// if (key.matches(filter)) {
		// seriesKeys.add(key);
		// seriesMap.put(key, new Integer(timeline.getSeriesKeys()
		// .indexOf(key)));
		// }
		// }
	}

	public int getItemCount() {
		int steps = timeline.getSteps();
		// log.debug("ItemCount is: {}", steps);
		return steps;
	}

	public DomainOrder getDomainOrder() {
		return DomainOrder.ASCENDING;
	}

	public int getItemCount(int arg0) {
		int steps = timeline.getSteps();
		// log.debug("Items for series {} are: {}", arg0, steps);
		return steps;
	}

	public Number getX(int arg0, int arg1) {
		return getXValue(arg0, arg1);
	}

	public double getXValue(int series, int step) {
		// log.debug("Returning x-value for series {}, step {}", series, step);
		Date date = new Date(timeline.getTimestamp(step));
		// date = new Date(timeline.getTimestamp(step)); //
		// timeline.getStart().getTime()
		// + step *
		// timeline.getStepSize()
		// );
		// log.debug("value for step {} is {}", step, date);
		// log.debug("   => " + date.getTime());
		return date.getTime();
	}

	public Number getY(int arg0, int arg1) {
		// log.debug("Returning y-value for {}, {}", arg0, arg1);
		Double val = getYValue(arg0, arg1);
		if (Double.isNaN(val))
			return null;
		return val;
	}

	public double getYValue(int series, int step) {
		// log.debug("looking up y-value for series {}, step {}", series, step);
		String key = seriesKeys.get(series);
		// log.debug("   series-key is: '{}'", key);
		Statistics st = timeline.get(step);
		// log.debug("   statistics for step {} are: {}", step, st);
		if (key == null || st == null || st.get(key) == null
				|| st.get(key) == 0.0) {
			// Statistics last = timeline.get(step - 1);
			// Statistics next = timeline.get(step + 1);
			// log.info("Trying to interpolate {} and {}", last, next);
			// if (last != null & next != null) {
			//
			// Double v1 = last.get(key);
			// Double v2 = next.get(key);
			//
			// if (v1 != null && v2 != null) {
			// log.info("Interpolating step {}", step);
			// return (v1 + v2) / 2.0d;
			// }
			// }
			return 0;
		}
		Double val = st.get(key);
		// log.debug("y-value is: {}", val);
		if (val == null)
			return Double.NaN; // 0.0;

		return val;
	}

	public int getSeriesCount() {
		// log.debug("Series count is {}", seriesKeys.size());
		return this.seriesKeys.size();
	}

	@SuppressWarnings("rawtypes")
	public Comparable getSeriesKey(int arg0) {
		return seriesKeys.get(arg0);
	}

	@SuppressWarnings("rawtypes")
	public int indexOf(Comparable arg0) {
		return this.seriesKeys.indexOf(arg0);
	}

	public void addChangeListener(DatasetChangeListener arg0) {
		listener.add(arg0);
	}

	public void removeChangeListener(DatasetChangeListener arg0) {
		listener.remove(arg0);
	}

	public DatasetGroup getGroup() {
		return group;
	}

	public void setGroup(DatasetGroup arg0) {
		this.group = arg0;
	}

	public void add(Statistics st) {
		add(System.currentTimeMillis(), st);
	}

	public void add(Long time, Statistics st) {
		timeline.add(time, st);

		for (String key : st.keySet()) {
			if (!seriesKeys.contains(key))
				seriesKeys.add(key);
		}

		for (DatasetChangeListener l : listener) {
			l.datasetChanged(new DatasetChangeEvent(this, this));
		}
	}
}