package stream.dashboard;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

import stream.Data;
import stream.ProcessContext;
import stream.StatefulProcessor;
import stream.expressions.Condition;

public abstract class Widget extends JPanel implements MouseListener,
		StatefulProcessor {

	/** The unique class ID */
	private static final long serialVersionUID = 3235914639396581290L;

	final static Color ACTIVE_BORDER_COLOR = new Color(0xa0, 0xa0, 0xa0);
	final static Color INACTIVE_BORDER_COLOR = new Color(0xe0, 0xe0, 0xe0);
	final JLabel title = new JLabel();
	final JPanel header = new JPanel(new BorderLayout());
	protected final JPanel content = new JPanel(new BorderLayout());
	final JPanel icons = new JPanel(new FlowLayout());
	Condition condition;
	protected String titleString;
	protected ProcessContext context;

	public Widget() {
		setLayout(new BorderLayout());
		setBackground(Color.WHITE);

		title.setBorder(new EmptyBorder(4, 4, 4, 4));
		icons.add(new JButton("x"));
		icons.setVisible(false);

		header.add(title, BorderLayout.WEST);
		header.add(icons, BorderLayout.EAST);
		header.setBackground(new Color(0xef, 0xef, 0xef));
		header.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0,
				INACTIVE_BORDER_COLOR));
		add(header, BorderLayout.NORTH);

		content.setBorder(new EmptyBorder(5, 5, 5, 5));
		content.setBorder(null);
		content.setBackground(Color.WHITE);

		add(content, BorderLayout.CENTER);
		setBorder(new LineBorder(INACTIVE_BORDER_COLOR));

		Font f = title.getFont().deriveFont(11.0f);
		title.setForeground(Color.DARK_GRAY);
		title.setFont(f);

		setSize(320, 240);
		// this.addMouseListener(this);
		// this.header.addMouseListener(this);
	}

	public Widget(int width, int height) {
		this();
		this.setSize(width, height);
	}

	public JPanel getHeader() {
		return header;
	}

	public void setTitle(String str) {
		title.setText(str);
		titleString = str;
	}

	public void setContent(Component c) {
		content.removeAll();
		content.add(c, BorderLayout.CENTER);
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
		setBorder(new LineBorder(ACTIVE_BORDER_COLOR));

		if (e.getSource() == header) {
			icons.setVisible(true);
		}
	}

	public void mouseExited(MouseEvent e) {
		setBorder(new LineBorder(INACTIVE_BORDER_COLOR));
		if (e.getSource() == header) {
			icons.setVisible(false);
		}
	}

	public void highlight(boolean b) {
		if (b) {
			setBorder(new LineBorder(ACTIVE_BORDER_COLOR));
		} else
			setBorder(new LineBorder(INACTIVE_BORDER_COLOR));
	}

	public void init(ProcessContext ctx) throws Exception {
		this.context = ctx;
	}

	public Condition getCondition() {
		return condition;
	}

	public void setCondition(Condition condition) {
		this.condition = condition;
	}

	public boolean handles(Data item) {
		if (condition == null) {
			return true;
		}

		return condition.matches(context, item);
	}

	/**
	 * @see stream.Processor#process(stream.Data)
	 */
	public abstract Data process(Data input);

	/**
	 * @see stream.StatefulProcessor#resetState()
	 */
	public void resetState() throws Exception {
	}

	/**
	 * @see stream.StatefulProcessor#finish()
	 */
	public void finish() throws Exception {
	}
}
