package stream.dashboard;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class WidgetMover extends MouseAdapter {

	static Logger log = LoggerFactory.getLogger(WidgetMover.class);
	final Dashboard dashboard;
	int gridx = 25;
	int gridy = 25;
	int offx = 0;
	int offy = 0;
	Widget selected = null;

	public WidgetMover(Dashboard dashboard) {
		this.dashboard = dashboard;
		gridx = dashboard.gridSize();
		gridy = dashboard.gridSize();
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		Component c = dashboard.display.getComponentAt(e.getPoint());
		if (c != null && c instanceof Widget) {
			Widget w = (Widget) c;
			w.highlight(true);
			dashboard.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
			if (w.getHeader().contains(e.getPoint())) {
			} else {
			}

		} else {
			dashboard.setCursor(Cursor.getDefaultCursor());
			for (Widget w : dashboard.getWidgets()) {
				w.highlight(false);
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// log.info("Mouse released at {}", e.getPoint());
		selected = null;
		dashboard.message(null);
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		// log.info("mouse dragged: {}", e);
		if (selected != null) {
			// log.info("Moving selected widget {} to {}", selected,
			// e.getPoint());
			int x = e.getX() + offx;
			int y = e.getY() + offy;
			x = gridx + x - (x % gridx);
			y = gridy + y - (y % gridy);

			selected.setLocation(x, y);
			dashboard.message("( " + selected.getLocation().x + " | "
					+ selected.getLocation().y + " )");
			return;
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {

		Component c = dashboard.display.getComponentAt(e.getPoint());
		if (c != null && (c instanceof Widget)) {
			selected = (Widget) c;
			// log.info("Found widget {} for {}", selected, e.getPoint());
			offx = selected.getX() - e.getX();
			offy = selected.getY() - e.getY();
			return;
		} else {
			// log.info("No widget found for point {}", e.getPoint());
		}
	}
}
