package stream.dashboard.renderer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.text.SimpleDateFormat;

import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import stream.dashboard.Message;

public class MessageRenderer extends DefaultListCellRenderer {

	static Logger log = LoggerFactory.getLogger(MessageRenderer.class);

	/** The unique class ID */
	private static final long serialVersionUID = 6075399482778842473L;

	final SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy");
	final SimpleDateFormat timeFormat = new SimpleDateFormat("hh:mm:ss");
	final Font dateTimeFont;

	public MessageRenderer() {
		dateTimeFont = new JLabel().getFont().deriveFont(10.0f);
	}

	public Component getListCellRendererComponent(JList<?> list, Object obj,
			int index, boolean isSelected, boolean cellHasFocus) {

		Message value = (Message) obj;
		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(new EmptyBorder(2, 2, 2, 2));
		panel.setBackground(Color.WHITE);

		JPanel datePanel = new JPanel();
		datePanel.setBackground(Color.WHITE);
		datePanel.setLayout(new BoxLayout(datePanel, BoxLayout.PAGE_AXIS));
		datePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		JLabel dateLabel = new JLabel(dateFormat.format(value.getTime()));
		dateLabel.setFont(dateTimeFont);
		dateLabel.setForeground(new Color(0x4, 0x4, 0x4));
		dateLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		JLabel timeLabel = new JLabel(timeFormat.format(value.getTime()));
		timeLabel.setFont(dateTimeFont);
		timeLabel.setForeground(new Color(0x4, 0x4, 0x4));
		timeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		datePanel.add(dateLabel);
		datePanel.add(timeLabel);

		JLabel text = new JLabel();
		text.setText(value.getText());
		text.setFont(dateTimeFont);
		text.setForeground(new Color(0x4, 0x4, 0x4));
		text.setBorder(new EmptyBorder(4, 4, 4, 4));

		panel.add(datePanel, BorderLayout.WEST);
		panel.add(text, BorderLayout.CENTER);

		log.info("Returning list-cell-renderer {}", panel);
		return panel;
	}
}
